# -*- coding: utf-8 -*-
from django.dispatch import Signal

setup_test_environment = Signal()
teardown_test_environment = Signal()

before_suite_run = Signal()
after_suite_run = Signal()

build_suite = Signal(providing_args=["suite"])

test_failure = Signal(providing_args=['test', 'err'])
test_error = Signal(providing_args=['test', 'err'])
test_success = Signal(providing_args=['test'])
test_skip = Signal(providing_args=['test', 'reason'])
test_expected_failure = Signal(providing_args=['test', 'err'])
test_unexpected_success = Signal(providing_args=['test'])



# -*- coding: utf-8 -*-
from optparse import make_option
from django_jenkins.management.commands import TaskListCommand


class Command(TaskListCommand):
    help = "Run flake8 over project apps"
    args = '[appname ...]'
    option_list = TaskListCommand.option_list + (
        make_option(
            '--flake8-file-output',
            action='store_true',
            dest='flake8_file_output',
            default=False,
            help='Store flake8 report in file'),
    )

    def get_task_list(self):
        return ('django_jenkins.tasks.run_flake8',)



# -*- coding: utf-8 -*-
from optparse import make_option
from django.conf import settings
from django.utils.importlib import import_module
from django_jenkins.management.commands import TaskListCommand


class Command(TaskListCommand):
    help = "Run django test suite using jenkins test runner"
    args = '[appname ...]'
    option_list = TaskListCommand.option_list + (
        make_option('--with-reports', action='store_true',
                    dest='with_reports', default=False,
                help='Create xunit reports files'),
        make_option("--coverage-html-report",
                    dest="coverage_html_report_dir",
                    default="",
                help="Enables code coverage and creates html coverage report"),
    )

    def get_tasks(self, *test_labels, **options):
        if options.get('coverage_html_report_dir',
                       getattr(settings, 'COVERAGE_HTML_REPORT', False)):
            self.tasks_cls.append(
                    import_module('django_jenkins.tasks.with_coverage').Task)
        return [task_cls(test_labels, options) for task_cls in self.tasks_cls]

    def get_task_list(self):
        enabled_tasks = getattr(settings, 'JENKINS_TASKS', ())

        tasks = []

        if 'django_jenkins.tasks.with_local_celery' in enabled_tasks:
            tasks.append('django_jenkins.tasks.with_local_celery')

        return tasks



# -*- coding: utf-8 -*-
import os
import sys
import pep8

from optparse import make_option
from django.conf import settings

from django_jenkins.functions import relpath
from django_jenkins.tasks import BaseTask, get_apps_locations


class Task(BaseTask):
    option_list = [
        make_option("--pep8-exclude",
                    dest="pep8-exclude",
                    default=pep8.DEFAULT_EXCLUDE + ",migrations",
                    help="exclude files or directories which match these "
                    "comma separated patterns (default: %s)" %
                    pep8.DEFAULT_EXCLUDE),
        make_option("--pep8-select", dest="pep8-select",
                    help="select errors and warnings (e.g. E,W6)"),
        make_option("--pep8-ignore", dest="pep8-ignore",
                    help="skip errors and warnings (e.g. E4,W)"),
        make_option("--pep8-max-line-length",
                    dest="pep8-max-line-length", type='int',
                    help="set maximum allowed line length (default: %d)" %
                    pep8.MAX_LINE_LENGTH),
        make_option("--pep8-rcfile", dest="pep8-rcfile",
                    help="PEP8 configuration file"),
    ]

    def __init__(self, test_labels, options):
        super(Task, self).__init__(test_labels, options)
        self.test_all = options['test_all']

        if options.get('pep8_file_output', True):
            output_dir = options['output_dir']
            if not os.path.exists(output_dir):
                os.makedirs(output_dir)
            self.output = open(os.path.join(output_dir, 'pep8.report'), 'w')
        else:
            self.output = sys.stdout

        self.pep8_rcfile = options['pep8-rcfile'] or Task.default_config_path()
        self.pep8_options = {'exclude': options['pep8-exclude'].split(',')}
        if options['pep8-select']:
            self.pep8_options['select'] = options['pep8-select'].split(',')
        if options['pep8-ignore']:
            self.pep8_options['ignore'] = options['pep8-ignore'].split(',')
        if options['pep8-max-line-length']:
            self.pep8_options['max_line_length'] = options['pep8-max-line-length']

    def teardown_test_environment(self, **kwargs):
        locations = get_apps_locations(self.test_labels, self.test_all)

        class JenkinsReport(pep8.BaseReport):
            def error(instance, line_number, offset, text, check):
                code = super(JenkinsReport, instance).error(line_number, offset, text, check)

                if not code:
                    return
                sourceline = instance.line_offset + line_number
                self.output.write('%s:%s:%s: %s\n' %
                                  (instance.filename, sourceline, offset + 1, text))

        pep8style = pep8.StyleGuide(parse_argv=False, config_file=self.pep8_rcfile,
                                    reporter=JenkinsReport,
                                    **self.pep8_options)

        for location in locations:
            pep8style.input_dir(relpath(location))

        self.output.close()

    @staticmethod
    def default_config_path():
        rcfile = getattr(settings, 'PEP8_RCFILE', 'pep8.rc')
        return rcfile if os.path.exists(rcfile) else None



# -*- coding: utf-8 -*-
import os
import re
import sys
from optparse import make_option
from pyflakes.scripts import pyflakes
from django_jenkins.functions import relpath
from django_jenkins.tasks import BaseTask, get_apps_locations

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO


class Task(BaseTask):
    option_list = [
        make_option("--pyflakes-with-migrations",
                    action="store_true", default=False,
                    dest="pyflakes_with_migrations",
                    help="Don't check migrations with pyflakes.")]

    def __init__(self, test_labels, options):
        super(Task, self).__init__(test_labels, options)
        self.test_all = options['test_all']
        self.with_migrations = options['pyflakes_with_migrations']

        if options.get('pyflakes_file_output', True):
            output_dir = options['output_dir']
            if not os.path.exists(output_dir):
                os.makedirs(output_dir)
            self.output = open(os.path.join(output_dir,
                                            'pyflakes.report'), 'w')
        else:
            self.output = sys.stdout

    def teardown_test_environment(self, **kwargs):
        locations = get_apps_locations(self.test_labels, self.test_all)

        # run pyflakes tool with captured output
        old_stdout, pyflakes_output = sys.stdout, StringIO()
        sys.stdout = pyflakes_output
        try:
            for location in locations:
                if os.path.isdir(location):
                    for dirpath, dirnames, filenames in \
                                        os.walk(relpath(location)):
                        if not self.with_migrations and \
                                dirpath.endswith(os.sep + 'migrations'):
                            continue
                        for filename in filenames:
                            if filename.endswith('.py'):
                                pyflakes.checkPath(os.path.join(dirpath,
                                                                filename))
                else:
                    pyflakes.checkPath(relpath(location))
        finally:
            sys.stdout = old_stdout

        # save report
        pyflakes_output.seek(0)

        while True:
            line = pyflakes_output.readline()
            if not line:
                break
            message = re.sub(r': ', r': [E] PYFLAKES:', line)
            self.output.write(message)

        self.output.close()



# -*- coding: utf-8; mode: django -*-
import os
import subprocess
import sys
import fnmatch
import codecs
from optparse import make_option
from django.conf import settings
from django_jenkins.functions import CalledProcessError
from django_jenkins.tasks import BaseTask, get_apps_locations


class Task(BaseTask):
    option_list = [
        make_option("--csslint-with-staticdirs",
                    dest="csslint_with-staticdirs",
                    default=False, action="store_true",
                    help="Check css files located in STATIC_DIRS settings"),
        make_option("--csslint-with-mincss",
                    dest="csslint_with_mincss",
                    default=False, action="store_true",
                    help="Do not ignore .min.css files"),
        make_option("--csslint-exclude",
                    dest="csslint_exclude", default="",
                    help="Exclude patterns"),
        make_option("--csslint-static-dirname",
                    dest="csslint_static-dirname", default="static",
                    help="Name of dir with css static files"),
        make_option("--csslint-ignore",
                    dest="csslint_ignore", default="",
                    help="Ignore rules")]

    def __init__(self, test_labels, options):
        super(Task, self).__init__(test_labels, options)
        self.test_all = options['test_all']
        self.to_file = options.get('csslint_file_output', True)
        self.with_static_dirs = options.get('csslint_with-staticdirs', False)
        self.csslint_with_minjs = options.get('csslint_with_mincss', False)

        if self.to_file:
            output_dir = options['output_dir']
            if not os.path.exists(output_dir):
                os.makedirs(output_dir)
            self.output = codecs.open(os.path.join(output_dir, 'csslint.report'), 'w', 'utf-8')
        else:
            self.output = sys.stdout

        self.exclude = options['csslint_exclude'].split(',')
        self.ignore = options['csslint_ignore']
        self.static_dirname = options.get('csslint_static-dirname', 'static')

    def teardown_test_environment(self, **kwargs):
        files = [path for path in self.static_files_iterator()]
        if self.to_file:
            fmt = 'lint-xml'
        else:
            fmt = 'text'

        if files:
            cmd = ['csslint', '--format=%s' % fmt] + files

            if self.ignore:
                cmd += ['--ignore=%s' % self.ignore]

            process = subprocess.Popen(cmd, stdout=subprocess.PIPE)
            output, err = process.communicate()
            retcode = process.poll()
            if retcode not in [0, 1]:  # normal csslint return codes
                raise CalledProcessError(retcode, cmd,
                                         output=output + '\n' + err)

            self.output.write(output.decode('utf-8'))
        elif self.to_file:
            self.output.write('<?xml version="1.0" encoding='
                              '"utf-8"?><lint></lint>')

        self.output.close()

    def static_files_iterator(self):
        locations = get_apps_locations(self.test_labels, self.test_all)

        def in_tested_locations(path):
            if not self.csslint_with_minjs and path.endswith('.min.css'):
                return False

            for location in locations:
                if path.startswith(location):
                    return True
            if self.with_static_dirs:
                for location in list(settings.STATICFILES_DIRS):
                    if path.startswith(location):
                        return True
            return False

        def is_excluded(path):
            for pattern in self.exclude:
                if fnmatch.fnmatchcase(path, pattern):
                    return True
            return False

        if hasattr(settings, 'CSSLINT_CHECKED_FILES'):
            for path in settings.CSSLINT_CHECKED_FILES:
                yield path

        if 'django.contrib.staticfiles' in settings.INSTALLED_APPS:
            # use django.contrib.staticfiles
            from django.contrib.staticfiles import finders

            for finder in finders.get_finders():
                for path, storage in finder.list(self.exclude):
                    path = os.path.join(storage.location, path)
                    if path.endswith('.css') and in_tested_locations(path):
                        yield path
        else:
            # scan apps directories for static folders
            for location in locations:
                for dirpath, dirnames, filenames in \
                                os.walk(os.path.join(location, self.static_dirname)):
                    for filename in filenames:
                        path = os.path.join(dirpath, filename)
                        if filename.endswith('.css') and \
                             in_tested_locations(path) and not \
                                 is_excluded(path):
                            yield path



# -*- coding: utf-8 -*-
# pylint: disable=W0201
import os
import sys
from optparse import make_option
from django.conf import settings
from django_jenkins.tasks import BaseTask, get_apps_under_test

from pylint import lint
from pylint.reporters.text import ParseableTextReporter


class Task(BaseTask):
    option_list = [make_option("--pylint-rcfile",
                               dest="pylint_rcfile",
                               help="pylint configuration file"),
                   make_option("--pylint-errors-only",
                               dest="pylint_errors_only",
                               action="store_true", default=False,
                               help="pylint output errors only mode")]

    def __init__(self, test_labels, options):
        super(Task, self).__init__(test_labels, options)

        self.test_all = options['test_all']
        self.config_path = options['pylint_rcfile'] or \
                                Task.default_config_path()
        self.errors_only = options['pylint_errors_only']

        if options.get('pylint_file_output', True):
            output_dir = options['output_dir']
            if not os.path.exists(output_dir):
                os.makedirs(output_dir)
            self.output = open(os.path.join(output_dir, 'pylint.report'), 'w')
        else:
            self.output = sys.stdout

    def teardown_test_environment(self, **kwargs):
        args = ["--rcfile=%s" % self.config_path]
        if self.errors_only:
            args += ['--errors-only']
        args += get_apps_under_test(self.test_labels, self.test_all)

        lint.Run(args, reporter=ParseableTextReporter(output=self.output),
                                                      exit=False)

        return True

    @staticmethod
    def default_config_path():
        rcfile = getattr(settings, 'PYLINT_RCFILE', 'pylint.rc')
        if os.path.exists(rcfile):
            return rcfile

        # use build-in
        root_dir = os.path.normpath(os.path.dirname(__file__))
        return os.path.join(root_dir, 'pylint.rc')



# -*- coding: utf-8; mode: django -*-
from django.conf import settings
from django_jenkins.tasks import BaseTask


class Task(BaseTask):
    """
    Run all celery tasks locally, not in a worker.
    """

    def setup_test_environment(self, **kwargs):
        settings.CELERY_ALWAYS_EAGER = True
        settings.CELERY_EAGER_PROPAGATES_EXCEPTIONS = True

